import { AppStateStatus, NativeEventSubscription } from 'react-native';
import { Barrier } from './Barrier';
import { STATES } from './constants';
import Semaphore from './Semaphore';
import Emitter from './Emitter';
export { STATES };
declare global {
    var __turboModuleProxy: object | undefined;
}
declare class StaticServer {
    _appStateSub?: NativeEventSubscription;
    _configPath?: string;
    _fileDir: string;
    _hostname: string;
    _nonLocal: boolean;
    _origin: string;
    _stopInBackground: boolean;
    _port: number;
    _signalBarrier?: Barrier<void>;
    _state: STATES;
    _stateChangeEmitter: Emitter;
    _id: number;
    _sem: Semaphore;
    get fileDir(): string;
    get hostname(): string;
    get nonLocal(): boolean;
    get origin(): string;
    get stopInBackground(): boolean;
    get port(): number;
    get state(): STATES;
    _setState(neu: STATES): void;
    /**
     * Creates a new StaticServer instance.
     *
     * Because the legacy, the following alternative signatures are supported,
     * and the overall constructor implementation is thus subpar.
     *
     * new StaticServer(port, root, opts);
     * new StaticServer(port, opts);
     * new StaticServer(port, root);
     * new StaticServer(opts);
     * new StaticServer(port);
     * new StaticServer();
     *
     * @param {object} options
     */
    constructor({ fileDir, nonLocal, port, stopInBackground, }: {
        fileDir: string;
        nonLocal?: boolean;
        port?: number;
        stopInBackground?: boolean;
    });
    addStateListener(listener: (newState: STATES) => void): () => void;
    _configureAppStateHandling(): void;
    _removeConfigFile(): Promise<void>;
    /**
     * This method throws if server is not in a "stable" state, i.e. not in one
     * of these: ACTIVE, CRASHED, INACTIVE.
     */
    _stableStateGuard(): void;
    start(): Promise<string>;
    /**
     * Soft-stop the server: if automatic pause for background is enabled,
     * it will be automatically reactivated when the app goes into foreground
     * again. End users are expected to use public .stop() method below, which
     * additionally cleans-up that automatic re-activation.
     * @returns {Promise<>}
     */
    _stop(): Promise<void>;
    /**
     * Stops or pauses the server, if it is running, depending on whether it was
     * started with keepAlive option or not (note, keepAlive means that server is
     * not paused / restarted when the app goes into background). Pausing the
     * server means it will
     * automatically start again the next time the app transitions from background
     * to foreground. To ensure the server is stopped for good, pass in `kill`
     * flag. In that case only explicit call to .start() will start the server
     * again.
     * @param kill
     * @returns {Promise<>}
     */
    stop(): Promise<void>;
    _handleAppStateChange(appState: AppStateStatus): void;
}
export default StaticServer;
/**
 * Extracts bundled assets into the specified regular directory,
 * preserving asset folder structure, and overwriting any conflicting files
 * in the destination.
 *
 * This is an Android-specific function; it does nothing if called on iOS.
 *
 * @param {string} [into=''] Optional. The destination folder for extracted
 *  assets. By default assets are extracted into the app's document folder.
 * @param {string} [from=''] Optional. Relative path of the root asset folder,
 *  starting from which all assets contained in that folder and its subfolders
 *  will be extracted into the destination folder, preserving asset folder
 *  structure. By default all bundled assets will be extracted.
 * @return {Promise} Resolves once unpacking is completed.
 */
export declare function extractBundledAssets(into?: string, from?: string): Promise<void>;
/**
 * Returns a server instance currently being in ACTIVE, STARTING,
 * or STOPPING state, if such server instance exists.
 * @return {StaticServer|undefined}
 */
export declare function getActiveServer(): StaticServer | undefined;
//# sourceMappingURL=index.d.ts.map